--- old/usr/src/cmd/fs.d/smbclnt/smbutil/view.c	Sun Mar 18 01:13:12 2018
+++ new/usr/src/cmd/fs.d/smbclnt/smbutil/view.c	Sun Mar 18 01:13:12 2018
@@ -34,6 +34,7 @@
 
 /*
  * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright 2013 Nexenta Systems, Inc.  All rights reserved.
  */
 
 #include <sys/types.h>
@@ -48,12 +49,9 @@
 
 #include <netsmb/smb.h>
 #include <netsmb/smb_lib.h>
-#include <netsmb/smb_netshareenum.h>
-
 #include "common.h"
 
-int enum_shares(smb_ctx_t *);
-void print_shares(int, int, struct share_info *);
+static int use_rap;
 
 void
 view_usage(void)
@@ -79,18 +77,27 @@
 	error = smb_ctx_scan_argv(ctx, argc, argv,
 	    SMBL_SERVER, SMBL_SERVER, USE_WILDCARD);
 	if (error)
-		return (error);
+		goto out;
 
 	error = smb_ctx_readrc(ctx);
 	if (error)
-		return (error);
+		goto out;
 
 	while ((opt = getopt(argc, argv, STDPARAM_OPT)) != EOF) {
 		if (opt == '?')
 			view_usage();
+		/*
+		 * This is an undocumented option, just for testing.
+		 * Use the old LanMan Remote API Protocol (RAP) for
+		 * enumerating shares.
+		 */
+		if (opt == 'B') {
+			use_rap++;
+			continue;
+		}
 		error = smb_ctx_opt(ctx, opt, optarg);
 		if (error)
-			return (error);
+			goto out;
 	}
 
 	smb_ctx_setshare(ctx, "IPC$", USE_IPC);
@@ -101,7 +108,7 @@
 	 */
 	error = smb_ctx_resolve(ctx);
 	if (error)
-		return (error);
+		goto out;
 
 	/*
 	 * Have server, share, etc. from above:
@@ -118,7 +125,7 @@
 	if (error) {
 		smb_error(gettext("//%s: login failed"),
 		    error, ctx->ct_fullserver);
-		return (error);
+		goto out;
 	}
 
 	error = smb_ctx_get_tree(ctx);
@@ -125,17 +132,19 @@
 	if (error) {
 		smb_error(gettext("//%s/%s: tree connect failed"),
 		    error, ctx->ct_fullserver, ctx->ct_origshare);
-		return (error);
+		goto out;
 	}
 
 	/*
 	 * Have IPC$ tcon, now list shares.
-	 * This prints its own errors.
+	 * Try RPC; if that fails, do RAP.
 	 */
-	error = enum_shares(ctx);
-	if (error)
-		return (error);
+	if (!use_rap)
+		error = share_enum_rpc(ctx, ctx->ct_fullserver);
+	if (error || use_rap)
+		error = share_enum_rap(ctx);
 
+out:
 	smb_ctx_free(ctx);
 	return (0);
 }
@@ -158,48 +167,29 @@
 };
 #endif
 
-int
-enum_shares(smb_ctx_t *ctx)
+/*
+ * Print one line of the share list, or
+ * if SHARE is null, print the header line.
+ */
+void
+view_print_share(char *share, int type, char *comment)
 {
-	struct share_info *share_info;
-	int error, entries, total;
+	char *stname;
+	int stindex;
 
-	/*
-	 * XXX: Later, try RPC first,
-	 * then fall back to RAP...
-	 */
-	error = smb_netshareenum(ctx, &entries, &total, &share_info);
-	if (error) {
-		smb_error(gettext("//%s failed to list shares"),
-		    error, ctx->ct_fullserver);
-		return (error);
+	if (share == NULL) {
+		printf(gettext("Share        Type       Comment\n"));
+		printf("-------------------------------\n");
+		return;
 	}
-	print_shares(entries, total, share_info);
-	return (0);
-}
 
-void
-print_shares(int entries, int total,
-	struct share_info *share_info)
-{
-	struct share_info *ep;
-	int i;
+	stindex = type & STYPE_MASK;
+	if (stindex > STYPE_UNKNOWN)
+		stindex = STYPE_UNKNOWN;
+	stname = gettext(shtype[stindex]);
 
-	printf(gettext("Share        Type       Comment\n"));
-	printf("-------------------------------\n");
+	if (comment == NULL)
+		comment = "";
 
-	for (ep = share_info, i = 0; i < entries; i++, ep++) {
-		int sti = ep->type & STYPE_MASK;
-		if (sti > STYPE_UNKNOWN)
-			sti = STYPE_UNKNOWN;
-		printf("%-12s %-10s %s\n", ep->netname,
-		    gettext(shtype[sti]),
-		    ep->remark ? ep->remark : "");
-		free(ep->netname);
-		free(ep->remark);
-	}
-	printf(gettext("\n%d shares listed from %d available\n"),
-	    entries, total);
-
-	free(share_info);
+	printf("%-12s %-10s %s\n", share, stname, comment);
 }
